#!/usr/bin/python3

import logging
import os
import random
import socket
import sys
import time
import unittest

develop_mode = False
if (develop_mode):
    sys.path.append(os.path.join(os.path.dirname(__file__), '..'))  # Find config and pamfax when no Package
    sys.path.append(os.path.join(os.path.dirname(__file__), '..', 'pamfax'))  # Find processors when no Package

from pamfax import PamFax

try:
    sys.path.append(os.path.join(os.path.dirname(__file__), '..'))
    from config import HOST, USERNAME, PASSWORD, APIKEY, APISECRET, DROPBOX_USERNAME, DROPBOX_PASSWORD
except:
    raise Exception('Copy config.example.py to config.py and adapt with your credentials first!')

IP_ADDR = socket.gethostbyname('www.airport1.de')

"""
Make sure to upload a file through

     https://sandbox-api.pamfax.biz/server/faxin/faxintest.php

before running this test.

Update 2020: This form seems to be outdated and may not work anymore as expected.
It's better you really send a fax to yourself (to your phone number, which should
be registered at PamFax), or ask s.o. at PamFax to do it for you?
That's why I changed the test code to proceed also if no faxes are in your inbox!

If you want to run the OnlineStorage tests you also have to authenticate yourself first
in a browser as described here:

    https://sandbox-apifrontend.pamfax.biz/processors/onlinestorage/
    
Do not forget to provide the usertoken, given e.g. by the method VerifyUser:

    https://api.pamfax.biz/OspAuth/DropboxStorage?usertoken=xxx

"""

# Set to either WARNING or DEBUG (warning: very much output!)
# logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger('pamfax')


def _assert_json(message, response, check_response=True):
    logger.debug(message)
    logger.debug(response)
    if check_response:
        assert response['result']['code'] == 'success'
    logger.debug('*' * 10)
    return response


def _assert_file(message, f, content_type):
    logger.debug(message)
    logger.debug(content_type)
    assert f is not None
    assert content_type is not None


# Seed our test account with credit and a fax
pamfax = PamFax(USERNAME, PASSWORD, host=HOST, apikey=APIKEY, apisecret=APISECRET)

message = 'Adding credit to sandbox user'
response = pamfax.add_credit_to_sandbox_user(1000, "Testing purposes")
_assert_json(message, response)

message = 'Listing inbox faxes'
response = pamfax.list_inbox_faxes()
_assert_json(message, response)

if 'content' not in response['InboxFaxes']:
    print("You have no faxes in your inbox - provide them first")
    file_uuid = None
    uuid = None
else:
    files = response['InboxFaxes']['content']
    f = files[0]
    file_uuid = f['file_uuid']
    uuid = f['uuid']


class TestPamFax(unittest.TestCase):
    """A set of unit tests for this implementation of the PamFax API."""

    # https://sandbox-apifrontend.pamfax.biz/processors/common/
    def test_Common(self):

        # New methods not implemented by dynaptico
        message = 'Getting currency by language code'
        response = pamfax.get_currency_by_lang('DE')
        _assert_json(message, response)

        message = 'Getting current culture info'
        response = pamfax.get_current_culture_info()
        _assert_json(message, response)

        message = 'Getting formatted price'
        response = pamfax.get_formatted_price('127.0.0.1')
        _assert_json(message, response)

        message = 'Listing cities'
        response = pamfax.list_cities('DEU')
        _assert_json(message, response)

        message = 'Listing countries prices'
        response = pamfax.list_countries_prices('EN')
        _assert_json(message, response)

        message = 'Listing country states'
        response = pamfax.list_country_states('US')
        _assert_json(message, response)

        message = 'Listing destinations for zone'
        response = pamfax.list_destinations_for_zone('1')
        _assert_json(message, response)

        message = 'Listing zip codes'
        response = pamfax.list_zip_codes('DEU', 'Berlin')
        _assert_json(message, response)

        message = 'Setting culture'
        response = pamfax.set_culture('DE')
        _assert_json(message, response)

        message = 'Getting current settings'
        response = pamfax.get_current_settings()
        _assert_json(message, response)

        # ToDo: find a solution to have at least 1 fax in inbox for the tests
        if file_uuid:
            message = 'Getting file'
            f, content_type = pamfax.get_file(file_uuid)
            _assert_file(message, f, content_type)

        message = 'Getting geo IP information'
        response = pamfax.get_geo_ip_information(IP_ADDR)
        _assert_json(message, response)

        # ToDo: find a solution to have at least 1 fax in inbox for the tests
        if uuid:
            message = 'Getting page preview'
            f, content_type = pamfax.get_page_preview(uuid, 1)
            _assert_file(message, f, content_type)

        message = 'Listing countries'
        response = pamfax.list_countries()
        _assert_json(message, response)

        message = 'Listing countries for zone'
        response = pamfax.list_countries_for_zone(1)
        _assert_json(message, response)

        message = 'Listing currencies 1'
        response = pamfax.list_currencies()
        _assert_json(message, response)

        message = 'Listing currencies 2'
        response = pamfax.list_currencies('JPY')
        _assert_json(message, response)

        message = 'Listing languages 1'
        response = pamfax.list_languages()
        _assert_json(message, response)

        message = 'Listing languages 2'
        response = pamfax.list_languages(50)
        _assert_json(message, response)

        message = 'Listing strings'
        response = pamfax.list_strings(['hello'])
        _assert_json(message, response)

        message = 'Listing supported file types'
        response = pamfax.list_supported_file_types()
        _assert_json(message, response)

        message = 'Listing timezones'
        response = pamfax.list_timezones()
        _assert_json(message, response)

        message = 'Listing versions'
        response = pamfax.list_versions()
        _assert_json(message, response)

        message = 'Listing zones'
        response = pamfax.list_zones()
        _assert_json(message, response)

    # https://sandbox-apifrontend.pamfax.biz/processors/faxhistory/
    def test_FaxHistory(self):
        message = 'Adding note to fax'
        response = pamfax.add_fax_note(uuid, 'This is my favorite fax')
        _assert_json(message, response)

        message = 'Counting faxes'
        response = pamfax.count_faxes('inbox')
        _assert_json(message, response)

        message = 'Deleting faxes'
        response = pamfax.delete_faxes([uuid])
        _assert_json(message, response)

        message = 'Restoring fax'
        response = pamfax.restore_fax(uuid)
        _assert_json(message, response)

        # message = 'Deleting faxes from trash'
        # response = pamfax.delete_faxes_from_trash([uuid])
        # _assert_json(message, response)

        message = 'Emptying trash'
        response = pamfax.empty_trash()
        _assert_json(message, response)

        message = 'Getting fax details'
        response = pamfax.get_fax_details(uuid)
        _assert_json(message, response)

        # message = 'Getting fax group'
        # response = pamfax.get_fax_group(uuid)
        # _assert_json(message, response)

        message = 'Getting inbox fax'
        response = pamfax.get_inbox_fax(uuid, mark_read=False)
        _assert_json(message, response)

        # message = 'Listing fax group'
        # response = pamfax.list_fax_group(uuid)
        # _assert_json(message, response)

        message = 'Listing fax notes'
        response = pamfax.list_fax_notes(uuid)
        _assert_json(message, response)

        message = 'Listing inbox faxes'
        response = pamfax.list_inbox_faxes()
        _assert_json(message, response)

        message = 'Listing outbox faxes'
        response = pamfax.list_outbox_faxes()
        _assert_json(message, response)

        message = 'Listing recent faxes'
        response = pamfax.list_recent_faxes()
        _assert_json(message, response)

        message = 'Listing sent faxes'
        response = pamfax.list_sent_faxes()
        _assert_json(message, response)

        out_uuid = response['SentFaxes']['content'][0]['uuid']

        message = 'Getting transmission report'
        f, content_type = pamfax.get_transmission_report(out_uuid)
        _assert_file(message, f, content_type)

        # message = 'Listing trash'
        # response = pamfax.list_trash()
        # _assert_json(message, response)

        message = 'Listing unpaid faxes'
        response = pamfax.list_unpaid_faxes()
        _assert_json(message, response)

        message = 'Setting fax as read'
        response = pamfax.set_fax_read(uuid)
        _assert_json(message, response)

        message = 'Setting faxes as read'
        response = pamfax.set_faxes_as_read([uuid])
        _assert_json(message, response)

        message = 'Setting spam state for faxes'
        response = pamfax.set_spam_state_for_faxes([uuid], is_spam=False)
        _assert_json(message, response)

    # https://sandbox-apifrontend.pamfax.biz/processors/faxjob/
    def test_FaxJob(self):
        message = 'Creating a fax job'
        response = pamfax.create()
        _assert_json(message, response)

        message = 'Listing available covers'
        response = pamfax.list_available_covers()
        _assert_json(message, response)

        message = 'Adding a cover'
        response = pamfax.set_cover(response['Covers']['content'][1]['id'], 'Dynaptico: Tomorrow On Demand')
        _assert_json(message, response)

        message = 'Adding a remote file'
        response = pamfax.add_remote_file('https://s3.amazonaws.com/dynaptico/Dynaptico.pdf')
        _assert_json(message, response)

        message = 'Adding a local file'
        filepath = os.path.join(os.path.dirname(__file__), 'Dynaptico.pdf')
        response = pamfax.add_file(filepath)
        _assert_json(message, response)

        message = 'Removing a file'
        response = pamfax.remove_file(response['FaxContainerFile']['file_uuid'])
        _assert_json(message, response)

        message = 'Adding recipient 1'
        rnd_phone = str(random.randint(10000000, 99999999))
        response = pamfax.add_recipient('+49030' + rnd_phone)
        _assert_json(message, response)

        message = 'Adding recipient 2'
        rnd_phone = str(random.randint(10000000, 99999999))
        response = pamfax.add_recipient('+49030' + rnd_phone)
        _assert_json(message, response)

        message = 'Removing a recipient'
        response = pamfax.remove_recipient(response['FaxRecipient']['number'])
        _assert_json(message, response)

        message = 'Listing recipients'
        response = pamfax.list_recipients()
        _assert_json(message, response)

        message = 'Listing fax files'
        response = pamfax.list_fax_files()
        _assert_json(message, response)

        # Check state
        logger.debug('*' * 10)
        logger.debug('Checking state')
        t = 0
        while True:
            fax_state = pamfax.get_state()
            logger.debug(fax_state)
            if fax_state['FaxContainer']['state'] == 'ready_to_send':
                break
            time.sleep(2)
            t += 2
            logger.debug("%d seconds elapsed...", t)
        assert fax_state['FaxContainer']['state'] == 'ready_to_send'

        message = 'Preview the fax'
        response = pamfax.get_preview()
        _assert_json(message, response)

        message = 'Send the fax'
        response = pamfax.send()
        _assert_json(message, response)

        # This only works if you don't have enough credit now
        # message = 'Send the fax later'
        # response = pamfax.send_later()
        # _assert_json(message, response)

        # This only works after the fax has moved to the fax history
        # message = 'Cloning the fax'
        # response = pamfax.clone_fax(faxjob['FaxContainer']['uuid'])
        # _assert_json(message, response)

    # https://sandbox-apifrontend.pamfax.biz/processors/numberinfo/
    def test_NumberInfo(self):
        message = 'Getting number info'
        response = pamfax.get_number_info('+81362763902')
        _assert_json(message, response)

        message = 'Getting page price'
        response = pamfax.get_page_price('+81362763902')
        _assert_json(message, response)

    # You have to authenticate with a browser first, to do so do as written here:
    # https://sandbox-apifrontend.pamfax.biz/processors/onlinestorage/
    # Don't forget to provide the token you got from your storage provider, e.g. for Dropbox:
    # http://sandbox-api.pamfax.biz/OspAuth/DropboxStorage?usertoken=<your_received_token>
    def test_OnlineStorage(self):
        provider = "DropboxStorage"  # GoogleStorage|DropboxStorage|BoxnetStorage

        # Deactivated, so we don't have to re-authenticate again all the time
        # message = 'Dropping authentication'
        # response = pamfax.drop_authentication('DropBoxStorage')
        # _assert_json(message, response)

        message = 'Checking provider state'
        response = pamfax.check_provider_state(provider)
        _assert_json(message, response)

        message = 'Getting provider logo'
        f, content_type = pamfax.get_provider_logo(provider, 16)
        _assert_file(message, f, content_type)

        message = 'Listing folder contents'
        response = pamfax.list_folder_contents(provider)
        response = _assert_json(message, response, False)
        if response['result']['code'] == 'not_authenticated':
            print('The test for list_folder_contents is skipped, ' \
                  'as it seems you are not authenticated for provider ' + provider + '.\n')
            print('To do so follow the steps on: https://sandbox-apifrontend.pamfax.biz/processors/onlinestorage/')

        # We could also test OnlineStorage::AddFileFromOnlineStorage here.. IFF the user is authenticated AND we got a uuid for a
        # single file (NOT a folder) from OnlineStorage::ListFolderContents AND we created a fax with FaxJob::Create before

        message = 'Listing providers'
        response = pamfax.list_providers()
        _assert_json(message, response)

        # message = 'Setting auth token'
        # response = pamfax.set_auth_token('DropBoxStorage', token)
        # _assert_json(message, response)

    # https://sandbox-apifrontend.pamfax.biz/processors/session/
    def test_Session(self):
        message = 'Creating login identifier'
        response = pamfax.create_login_identifier(timetolifeminutes=10)
        _assert_json(message, response)

        message = 'Listing changes'
        response = pamfax.list_changes()
        _assert_json(message, response)

        # message = 'Logging out'
        # response = pamfax.logout()
        # _assert_json(message, response)

        message = 'Pinging'
        response = pamfax.ping()
        _assert_json(message, response)

        message = 'Registering listener'
        response = pamfax.register_listener(['faxsending', 'faxfailed'])
        _assert_json(message, response)

        message = 'Reloading user'
        response = pamfax.reload_user()
        _assert_json(message, response)

        message = 'Verifying user'
        response = pamfax.verify_user(USERNAME, PASSWORD)
        _assert_json(message, response)

    # https://sandbox-apifrontend.pamfax.biz/processors/shopping/
    def test_Shopping(self):
        # message = 'Getting invoice'
        # f, content_type = pamfax.get_invoice('')
        # _assert_file(message, f, content_type)

        message = 'Getting nearest fax in number'
        response = pamfax.get_nearest_fax_in_number(IP_ADDR)
        _assert_json(message, response)

        message = 'Getting shop link'
        response = pamfax.get_shop_link('pro_plan')
        _assert_json(message, response)

        message = 'Listing available items'
        response = pamfax.list_available_items()
        _assert_json(message, response)

        message = 'Listing fax in area codes'
        response = pamfax.list_fax_in_areacodes('JP')
        _assert_json(message, response)

        message = 'Listing fax in countries'
        response = pamfax.list_fax_in_countries()
        _assert_json(message, response)

        message = 'Redeeming credit voucher'
        response = pamfax.redeem_credit_voucher('PCPC0815')
        _assert_json(message, response)

    # https://sandbox-apifrontend.pamfax.biz/processors/userinfo/
    def test_UserInfo(self):
        # Can be executed only once, or would require to create randomized names & emails
        # message = 'Creating user'
        # response = pamfax.create_user(name='abc123xyz', username='abc123xyz@gmail.com', password='xyz123abc', email='abc123xyz@gmail.com', culture='en-US')
        # _assert_json(message, response)

        # Dangerous? Would it delete yourself?
        # message = 'Deleting user'
        # response = pamfax.delete_user()
        # _assert_json(message, response)

        message = 'Getting culture info'
        response = pamfax.get_culture_info()
        _assert_json(message, response)

        message = 'Has plan?'
        response = pamfax.has_plan()
        _assert_json(message, response)

        message = 'Listing expirations'
        response = pamfax.list_expirations()
        _assert_json(message, response)

        message = 'Listing inboxes'
        response = pamfax.list_inboxes(expired_too=True, shared_too=True)
        _assert_json(message, response)

        message = 'Listing orders'
        response = pamfax.list_orders()
        _assert_json(message, response)

        message = 'Listing profiles'
        response = pamfax.list_profiles()
        _assert_json(message, response)

        message = 'Listing user agents'
        response = pamfax.list_user_agents(max=2)
        _assert_json(message, response)

        message = 'Listing wall messages'
        response = pamfax.list_wall_messages(count=1)
        _assert_json(message, response)

        message = 'Sending message'
        response = pamfax.send_message('Hello, world!', type='email', recipient='test@example.com', subject='Hello')
        _assert_json(message, response)

        # Commented just to prevent all the reset password emails
        # message = 'Sending password reset message'
        # response = pamfax.send_password_reset_message(USERNAME)
        # _assert_json(message, response)

        message = 'Setting online storage settings'
        response = pamfax.set_online_storage_settings('DropBoxStorage', ['inbox_enabled=1', 'inbox_path=/'])
        _assert_json(message, response)

        # message = 'Setting password'
        # response = pamfax.set_password(PASSWORD, hashFunction='plain')
        # _assert_json(message, response)

        # message = 'Setting profile properties'
        # response = pamfax.set_profile_properties(profile='test', properties=)
        # _assert_json(message, response)

        # Requires an email
        message = 'Validating new username'
        response = pamfax.validate_new_username('kaninchen@hoppel.de')
        _assert_json(message, response)


if __name__ == '__main__':

    run_all = False

    # Either run all tests or en- or disable test packages by (un)commenting
    if run_all:
        unittest.main()
    else:
        suite = unittest.TestSuite()

        suite.addTest(TestPamFax("test_Common"))
        suite.addTest(TestPamFax("test_FaxHistory"))
        suite.addTest(TestPamFax("test_FaxJob"))
        suite.addTest(TestPamFax("test_NumberInfo"))
        suite.addTest(TestPamFax("test_OnlineStorage"))
        suite.addTest(TestPamFax("test_Session"))
        suite.addTest(TestPamFax("test_Shopping"))
        suite.addTest(TestPamFax("test_UserInfo"))

        runner = unittest.TextTestRunner()
        runner.run(suite)



#!/usr/bin/python3

"""
The classes defined in this module should correspond 1-to-1* to the
specifications in the PamFax API docs. See the link below for more info:

https://sandbox-apifrontend.pamfax.biz/summary/processors/

Users should not need to instantiate this class in applications using PamFax.
Instead, just call the action directly on the PamFax object, and it will be
delegated to the correct processor automatically.

NOTE: This module is for and has only been tested under Python 3.6+.

Following signatures have changed to the older (Python 2) implementation:

_get_and_check_response
_get
_post
"""

import json
import logging
import os
import socket
from urllib.parse import urlencode

import requests

IP_ADDR = socket.gethostbyname(socket.gethostname())
USER_AGENT = 'a1pamfax'
ORIGIN = 'script'
CONTENT_TYPE = 'content-type'
CONTENT_TYPE_JSON = 'application/json'

logger = logging.getLogger('pamfax')

https_session = requests.session()  # Re-use this session always


# ----------------------------------------------------------------------------
# "private" helper methods
# ----------------------------------------------------------------------------

def _get_url(base_url, action, api_credentials, **kwargs):
    """Construct the URL that corresponds to a given action.

    Arguments:
    base_url -- The base of the URL corresponding to the PamFax processor name
    action -- The PamFax method to append to the base URL
    api_credentials -- The API credentials including user token extracted from /Session/VerifyUser

    Keyword arguments:
    **kwargs -- optional HTTP parameters to send to the PamFax URL

    """
    url = '%s/%s%s' % (base_url, action, api_credentials)
    if not kwargs:
        return url
    query = dict()
    for arg in kwargs:
        kwarg = kwargs[arg]
        if kwarg:
            if isinstance(kwarg, list):
                for i in range(0, len(kwarg)):
                    query['%s[%d]' % (arg, i)] = kwarg[i]
            else:
                query[arg] = kwarg
    url = '%s&%s' % (url, urlencode(query))
    return url


def _get_and_check_response(method, host, url, body=None, headers=None, files=None, data=None, timeout=30):
    """Wait for the HTTPS response and throw an exception if the return
    status is not OK. Return either a dict based on the
    HTTP response in JSON, or if the response is not in JSON format,
    return a tuple containing the data in the body and the content type.
    """
    url = 'https://' + host + url
    # print(url)
    if files:
        res = https_session.post(url, files=files, data=data, timeout=timeout)
    elif method == 'POST':
        res = https_session.post(url, body, headers, timeout=timeout)
    else:
        res = https_session.get(url, timeout=timeout)
    res.raise_for_status()
    content_type = res.headers.get(CONTENT_TYPE, None)
    content = res.text
    if content_type and content_type.startswith(CONTENT_TYPE_JSON):
        # Quickfix to remove second key in bad API response
        key = '"FaxContainerFile":'
        if content.count(key) == 2:
            content = content[:content.rfind(key)].rstrip(',') + "}"
        return json.loads(content)
    else:
        return (content, content_type)


def _get(host, url):
    """Gets the specified url and returns the response."""
    logger.info("getting url '%s'", url)
    return _get_and_check_response('GET', host, url)


def _post(host, url, files, data):
    """Posts to the specified url and returns the response."""
    logger.info("posting to url '%s'", url)
    return _get_and_check_response('POST', host, url, files=files, data=data)


# ----------------------------------------------------------------------------
# Common
# ----------------------------------------------------------------------------

class Common:
    """Class encapsulating common actions for the PamFax API.
    New methods introduced AFTER dynaptico are marked as NEW, but
    startCreateZipArchive has still to be implemented. The new methods are:
    - GetCurrencyByLang
    - GetCurrentCultureInfo
    - GetFormattedPrice
    - ListCities
    - ListCountriesPrices
    - ListCountryStates
    - ListDestinationsForZone
    - ListZipCodes
    - SetCulture
    - startCreateZipArchive [ToDo]
    """

    def __init__(self, api_credentials, http):
        """Instantiates the Common class"""
        self.base_url = '/Common'
        self.api_credentials = api_credentials
        self.http = http

    def get_currency_by_lang(self, lang):
        """NEW Returns currency and format for language code.

        Arguments:
        lang -- ISO2 language code, e.g. DE
        """
        url = _get_url(self.base_url, 'GetCurrencyByLang', self.api_credentials, lang=lang)
        return _get(self.http, url)

    def get_current_culture_info(self):
        """NEW Returns the current culture info data."""
        url = _get_url(self.base_url, 'GetCurrentCultureInfo', self.api_credentials)
        return _get(self.http, url)

    def get_formatted_price(self, ip=None):
        """NEW Returns necessary format for later formation.

        Arguments:
        ip -- IP [optional]
        """
        url = _get_url(self.base_url, 'GetFormattedPrice', self.api_credentials, ip=ip)
        return _get(self.http, url)

    def list_cities(self, country_code_a3):
        """NEW Returns the list of all cities where ordering Fax Number is
        available in countries with stronger regulation rules (e.g. Germany).
        Returns NO_CITIES if specifying a city name is not necessary
        by regulation rules.

        Arguments:
        country_code_a3 -- Alpha3 country code, DEU for Germany
        """
        url = _get_url(self.base_url, 'ListCities', self.api_credentials, country_code_a3=country_code_a3)
        return _get(self.http, url)

    def list_countries_prices(self, language=None):
        """NEW Future replacement for ListCountriesForZone.
        Returns the list of all supported countries for fax sending grouped by prices.
        For example cost to BY = .06ec, RU = .08ec, DE = .02ec, US = .06ec
        A list will be returned with
        Group 1, price basic. pro, on demand: DE
        Group 2, price basic. pro, on demand: BY, US
        Group 3, price basic. pro, on demand: RU
        List will be sorted by prices ASC

        Arguments:
        language -- alpha-2 code for translation if needed [optional]
        """
        url = _get_url(self.base_url, 'ListCountriesPrices', self.api_credentials, language=language)
        return _get(self.http, url)

    def list_country_states(self, country_code=None):
        """NEW List states in country. If country is not passed, will assume US.

        Arguments:
        country_code -- ISO country code, e.g. US, RU
        """
        url = _get_url(self.base_url, 'ListCountryStates', self.api_credentials, country_code=country_code)
        return _get(self.http, url)

    def list_destinations_for_zone(self, zone=None):
        """NEW Returns all Destinations in the given zone.
        Result includes phone number pattern (countryprefix).

        Arguments:
        zone -- see list_zones, e.g. 1
        """
        url = _get_url(self.base_url, 'ListDestinationsForZone', self.api_credentials, zone=zone)
        return _get(self.http, url)

    def list_zip_codes(self, country_code_a3, city_name_or_id, exact=None):
        """NEW Returns all supported zip codes that are available in the supplied country and city.
        City value (id or name) should be from the Common::ListCities list.

        Arguments:
        country_code_a3 -- Alpha-3 country code, DEU for Germany
        city_name_or_id -- ID or CityName from Common::ListCities
        exact - use 1 if exact values needed, for example Berlin will be ignore codes for "Berlin, stadt" [optional]
        """
        url = _get_url(self.base_url, 'ListZipCodes', self.api_credentials, country_code_a3=country_code_a3,
                       city_name_or_id=city_name_or_id, exact=exact)
        return _get(self.http, url)

    def set_culture(self, language):
        """NEW Set culture for non-authorized users.
        API will return messages for non-authorized users:
        1. Auto-detect language (geo-ip, etc)
        2. Developer can set language by this function and API will not make auto-detects, valid on in current session.

        Arguments:
        language -- alpha-2 code for translation if needed [optional]
        """
        url = _get_url(self.base_url, 'SetCulture', self.api_credentials, language=language)
        return _get(self.http, url)

    def get_current_settings(self):
        """Returns the current settings for timezone and currency.
        This is the format/timezone ALL return values of the API are in. These are taken from the user
        (if logged in, the api user's settings or the current ip address)

        """
        url = _get_url(self.base_url, 'GetCurrentSettings', self.api_credentials)
        return _get(self.http, url)

    def get_file(self, file_uuid):
        """Returns file content.

        Will return binary data and headers that give the filename and mimetype.
        Note: User identified by usertoken must be owner of the file or
        the owner must have shared this file to the requesting user.
        Share dependencies are resolved (dependend on the type) via the
        fax_inbox, fax_history_files or user_covers table.

        Arguments:
        file_uuid -- The uuid of the file to get

        """
        url = _get_url(self.base_url, 'GetFile', self.api_credentials, file_uuid=file_uuid)
        return _get(self.http, url)

    def get_geo_ip_information(self, ip):
        """Returns Geo information based on the given IP address (IPV4)

        Arguments:
        ip -- the ip to get geo information off

        """
        url = _get_url(self.base_url, 'GetGeoIPInformation', self.api_credentials, ip=ip)
        return _get(self.http, url)

    def get_page_preview(self, uuid, page_no, max_width=None, max_height=None):
        """Returns a preview page for a fax.

        May be in progress, sent or from inbox.

        Arguments:
        uuid -- The uuid of the fax to get preview for
        page_no -- Page number to get (1,2,...)
        max_width -- Maximum width in Pixel
        max_height -- Maximum height in Pixel

        """
        url = _get_url(self.base_url, 'GetPagePreview', self.api_credentials, uuid=uuid, page_no=page_no,
                       max_width=max_width, max_height=max_height)
        return _get(self.http, url)

    def list_countries(self, culture=None):
        """Returns all countries with their translated names and the default zone"""
        url = _get_url(self.base_url, 'ListCountries', self.api_credentials, culture=culture)
        return _get(self.http, url)

    def list_countries_for_zone(self, zone):
        """Returns all countries in the given zone

        Result includes their translated names, countrycode and country-prefix.

        Arguments:
        zone -- Zone of the country which is wanted (1-7)

        """
        url = _get_url(self.base_url, 'ListCountriesForZone', self.api_credentials, zone=zone)
        return _get(self.http, url)

    def list_currencies(self, code=None):
        """Returns the list of supported currencies.

        Result contains convertion rates too.
        If code is given will only return the specified currency's information.

        Keyword arguments:
        code -- CurrencyCode

        """
        url = _get_url(self.base_url, 'ListCurrencies', self.api_credentials, code=code)
        return _get(self.http, url)

    def list_languages(self, min_percent_translated=None):
        """List all available languages.

        Result may be filtered tso that only languages are returned that are
        at least translated $min_percent_translated %

        Keyward arguments:
        min_percent_translated -- the percentage value the languages have to be translated

        """
        url = _get_url(self.base_url, 'ListLanguages', self.api_credentials,
                       min_percent_translated=min_percent_translated)
        return _get(self.http, url)

    def list_strings(self, ids, culture=None):
        """Returns a list of strings translated into the given language.

        Arguments:
        ids -- array of String identifiers. You may also pass a comma separated list as $ids[0] (ids[0]=BTN_YES,BTN_NO).

        Keyword arguments:
        culture -- culture identifier, defaults to users culture. Accepts full culture-codes like en-US, de-DE and just a language code like en, de, ...

        """
        url = _get_url(self.base_url, 'ListStrings', self.api_credentials, ids=ids, culture=culture)
        return _get(self.http, url)

    def list_supported_file_types(self):
        """Returns the supported file types for documents that can be faxed."""
        url = _get_url(self.base_url, 'ListSupportedFileTypes', self.api_credentials)
        return _get(self.http, url)

    def list_timezones(self):
        """List all supported timezones"""
        url = _get_url(self.base_url, 'ListTimezones', self.api_credentials)
        return _get(self.http, url)

    def list_versions(self):
        """Lists the current Versions.
        Result contains versions for the PamFax Gadget, Client etc and returns
        the version and update url
        """
        url = _get_url(self.base_url, 'ListVersions', self.api_credentials)
        return _get(self.http, url)

    def list_zones(self):
        """Returns price and price_pro for a given zone"""
        url = _get_url(self.base_url, 'ListZones', self.api_credentials)
        return _get(self.http, url)


# ----------------------------------------------------------------------------
# FaxHistory
# ----------------------------------------------------------------------------

class FaxHistory:
    """Class encapsulating actions related to fax history for this account"""

    def __init__(self, api_credentials, http):
        """Instantiates the FaxHistory class"""
        self.base_url = '/FaxHistory'
        self.api_credentials = api_credentials
        self.http = http

    def add_fax_note(self, fax_uuid, note):
        """Add a note (free text) to the fax

        Arguments:
        fax_uuid -- uuid of the fax
        note -- text to add to the fax

        """
        url = _get_url(self.base_url, 'AddFaxNote', self.api_credentials, fax_uuid=fax_uuid, note=note)
        return _get(self.http, url)

    def count_faxes(self, type):
        """Returns the number of faxes from users history with a specific state.

        Arguments:
        type -- Possible values: history, inbox, inbox_unread, outbox or unpaid

        """
        url = _get_url(self.base_url, 'CountFaxes', self.api_credentials, type=type)
        return _get(self.http, url)

    def delete_fax(self):
        """DEPRECATED! Use DeleteFaxes instead"""
        return None

    def delete_fax_from_trash(self):
        """DEPRECATED! Use DeleteFaxesFromTrash instead"""
        return None

    def delete_faxes(self, uuids, siblings_too=None):
        """Moves faxes to the trash.

        If siblings_too is true will perform for the given faxes and all other recipients
        from the same fax jobs.
        siblings_too will only be evaluated for uuids beloging to an outgoing fax and will be
        ignored for incoming faxes uuids

        """
        url = _get_url(self.base_url, 'DeleteFaxes', self.api_credentials, uuids=uuids, siblings_too=siblings_too)
        return _get(self.http, url)

    def delete_faxes_from_trash(self, uuids):
        """Removes faxes from trash

        This method is similar to EmptyTrash() which deletes all the faxes from trash

        Arguments:
        uuids -- ids of faxes to be removed vom trash

        """
        url = _get_url(self.base_url, 'DeleteFaxesFromTrash', self.api_credentials, uuids=uuids)
        return _get(self.http, url)

    def empty_trash(self):
        """Removes all faxes from trash for user and if user is member of a company and has delete rights also for the owners inbox faxes"""
        url = _get_url(self.base_url, 'EmptyTrash', self.api_credentials)
        return _get(self.http, url)

    def get_fax_details(self, uuid):
        """Returns the details of a fax in progress.

        Arguments:
        uuid -- UUID of the fax to show

        """
        url = _get_url(self.base_url, 'GetFaxDetails', self.api_credentials, uuid=uuid)
        return _get(self.http, url)

    def get_fax_group(self, uuid):
        """Returns a fax groups details.

        Arguments:
        uuid -- Uuid of one of the faxes in the group.

        """
        url = _get_url(self.base_url, 'GetFaxGroup', self.api_credentials, uuid=uuid)
        return _get(self.http, url)

    def get_inbox_fax(self, uuid, mark_read=None):
        """Returns the details of a fax in the inbox.

        Arguments:
        uuid -- UUID of the fax to show
        mark_read -- If true marks the fax as read (default: false).

        """
        url = _get_url(self.base_url, 'GetInboxFax', self.api_credentials, uuid=uuid, mark_read=mark_read)
        return _get(self.http, url)

    def get_transmission_report(self, uuid):
        """Get a .pdf-Version of a transmission report.

        On the transmission report basic data of the fax and a preview of the first page is shown.
        Should always be called with API_MODE_PASSTHRU, as the result is the pdf as binary data

        Arguments:
        uuid -- @attribute[RequestParam('uuid','string')]

        """
        url = _get_url(self.base_url, 'GetTransmissionReport', self.api_credentials, uuid=uuid)
        return _get(self.http, url)

    def list_fax_group(self, uuid, current_page=None, items_per_page=None):
        """Lists all faxes in a group (that are sent as on job).

        Arguments:
        uuid -- Uuid of one of the faxes in the group.

        Keyword arguments:
        current_page -- The page which should be shown
        items_per_page -- How many items are shown per page

        """
        url = _get_url(self.base_url, 'ListFaxGroup', self.api_credentials, uuid=uuid, current_page=current_page,
                       items_per_page=items_per_page)
        return _get(self.http, url)

    def list_fax_notes(self, fax_uuid):
        """Lists all notes for the given fax in reverse order (latest first)

        Arguments:
        fax_uuid -- uuid of the fax to list notes for

        """
        url = _get_url(self.base_url, 'ListFaxNotes', self.api_credentials, fax_uuid=fax_uuid)
        return _get(self.http, url)

    def list_inbox_faxes(self, current_page=None, items_per_page=None):
        """List all faxes in the inbox of the current user.

        Keyword arguments:
        current_page -- The page which should be shown
        items_per_page -- How many items are shown per page

        """
        url = _get_url(self.base_url, 'ListInboxFaxes', self.api_credentials, current_page=current_page,
                       items_per_page=items_per_page)
        return _get(self.http, url)

    def list_outbox_faxes(self, current_page=None, items_per_page=None):
        """Faxes in the outbox that are currently in the sending process

        Keyword arguments:
        current_page -- The page which should be shown
        items_per_page -- How many items are shown per page

        """
        url = _get_url(self.base_url, 'ListOutboxFaxes', self.api_credentials, current_page=current_page,
                       items_per_page=items_per_page)
        return _get(self.http, url)

    def list_recent_faxes(self, count=None, data_to_list=None):
        """Returns a list of latest faxes for the user.

        Does not contain deleted and delayed faxes (See ListTrash for deleted faxes).

        Keyword arguments:
        count -- The count of items to return. Valid values are between 1 and 100
        data_to_list -- Any message types you want this function to return. Allowed models are 'sent', 'inbox', 'outbox'. Leave empty to get faxes of any type.

        """
        url = _get_url(self.base_url, 'ListRecentFaxes', self.api_credentials, count=count, data_to_list=data_to_list)
        return _get(self.http, url)

    def list_sent_faxes(self, current_page=None, items_per_page=None):
        """List all sent faxes (successful or not)

        Keyword arguments:
        current_page -- The page which should be shown
        items_per_page -- How many items are shown per page

        """
        url = _get_url(self.base_url, 'ListSentFaxes', self.api_credentials, current_page=current_page,
                       items_per_page=items_per_page)
        return _get(self.http, url)

    def list_trash(self, current_page=None, items_per_page=None):
        """List all faxes in trash

        Keyword arguments:
        current_page -- The page which should be shown
        items_per_page -- How many items are shown per page

        """
        url = _get_url(self.base_url, 'ListTrash', self.api_credentials, current_page=current_page,
                       items_per_page=items_per_page)
        return _get(self.http, url)

    def list_unpaid_faxes(self, current_page=None, items_per_page=None):
        """Lists all unpaid faxes that are waiting for a payment.

        When this user makes a transaction to add credit, these faxes will be sent automatically
        if they are younger that 2 hours.

        Keyword arguments:
        current_page -- The page which should be shown
        items_per_page -- How many items are shown per page

        """
        url = _get_url(self.base_url, 'ListUnpaidFaxes', self.api_credentials, current_page=current_page,
                       items_per_page=items_per_page)
        return _get(self.http, url)

    def restore_fax(self, uuid):
        """Restores a fax from the trash.

        Arguments:
        uuid -- uuid of fax to restore

        """
        url = _get_url(self.base_url, 'RestoreFax', self.api_credentials, uuid=uuid)
        return _get(self.http, url)

    def set_fax_read(self, uuid):
        """Sets a fax' read date to current time.

        Fax needs to be a fax in the inbox.

        Arguments:
        uuid -- uuid of fax to set as read

        """
        url = _get_url(self.base_url, 'SetFaxRead', self.api_credentials, uuid=uuid)
        return _get(self.http, url)

    def set_faxes_as_read(self, uuids):
        """Sets the read date of all the faxes to the current time

        Arguments:
        uuids -- array of uuids for faxes to set as read

        """
        url = _get_url(self.base_url, 'SetFaxesAsRead', self.api_credentials, uuids=uuids)
        return _get(self.http, url)

    def set_spam_state_for_faxes(self, uuids, is_spam=None):
        """Sets the spamscore for all the faxes depending on the flag "is_spam"
        Takes an array of faxes UUIDs and marks them a spam if the second argument (is_spam) is true.
        Removes the Spam state if is_spam is false.
        If 15 or more faxes of the same sender has been marked as spam, all incoming faxes are directly moved to the trash.
        This is user specific, so if user A reports 15 faxes of one sender, then only all incoming faxes from the sender to
        him are directly sent to the trash.

        """
        url = _get_url(self.base_url, 'SetSpamStateForFaxes', self.api_credentials, uuids=uuids, is_spam=is_spam)
        return _get(self.http, url)


# ----------------------------------------------------------------------------
# FaxJob
# ----------------------------------------------------------------------------

class FaxJob:
    """Class encapsulating a specific fax job"""

    def __init__(self, api_credentials, http):
        """Instantiates the FaxJob class"""
        self.base_url = '/FaxJob'
        self.api_credentials = api_credentials
        self.http = http

    def add_file(self, filename, origin=None):
        """Adds a file to the current fax.

        Requires the file to be uploaded as POST parameter named 'file' as a standard HTTP upload. This could be either Content-type: multipart/form-data with file content as base64-encoded data or as Content-type: application/octet-stream with just the binary data.
        See http://www.faqs.org/rfcs/rfc1867.html for documentation on file uploads.

        Arguments:
        filename -- Name of the file. You can also use the same file name for each file (i.e "fax.pdf")

        Keyword arguments:
        origin -- Optional file origin (ex: photo, scan,... - maximum length is 20 characters).

        """
        with open(filename, 'rb') as file:
            content = file.read()
        files = {'file': content}
        basename = os.path.basename(filename)
        url = _get_url(self.base_url, 'AddFile', self.api_credentials, filename=basename, origin=origin)
        values = {'filename': basename}
        return _post(self.http, url, files, data=values)

    def add_file_from_online_storage(self, provider, uuid):
        """Add a file identified by an online storage identifier.

        You'll have to use the OnlineStorageApi to identify a user for an online storage provider first
        and then get listings of his files. These will contain the file identifiers used by this method.

        Arguments:
        provider -- Identifies the provider used (see OnlineStorageApi)
        uuid -- Identifies the file to be added (see OnlineStorageApi)

        """
        url = _get_url(self.base_url, 'AddFileFromOnlineStorage', self.api_credentials, provider=provider, uuid=uuid)
        return _get(self.http, url)

    def add_recipient(self, number, name=None):
        """Adds a recipient to the current fax."""
        url = _get_url(self.base_url, 'AddRecipient', self.api_credentials, number=number, name=name)
        return _get(self.http, url)

    def add_recipients(self, numbers, names=None):
        """Adds recipients to the current fax.
        The given recipients will be added to current recipients.

        """
        url = _get_url(self.base_url, 'AddRecipients', self.api_credentials, numbers=numbers, names=names)
        return _get(self.http, url)

    def add_remote_file(self, url):
        """Add a remote file to the fax.
        url may contain username:password for basic http auth, but this is the only supported
        authentication method.
        URL Examples:
        http://myusername:andpassord

        """
        url = _get_url(self.base_url, 'AddRemoteFile', self.api_credentials, url=url)
        return _get(self.http, url)

    def cancel(self, uuid, siblings_too=None):
        """Cancels fax sending for a fax recipient or a whole fax job.
        If siblings_too is true will cancel all faxes in the job the
        fax with uuid belongs to.

        """
        url = _get_url(self.base_url, 'Cancel', self.api_credentials, uuid=uuid, siblings_too=siblings_too)
        return _get(self.http, url)

    def clone_fax(self, uuid, user_ip=IP_ADDR, user_agent=USER_AGENT):
        """Clones an already sent fax in the API backend and returns it.

        Arguments:
        uuid -- The uuid of the source fax

        Keyword arguments:
        user_ip -- The IP address of the client (if available). Put your own IP address otherwise.
        user_agent -- User agent string of the client device. If not available, put something descriptive (like "iPhone OS 2.2")

        """
        url = _get_url(self.base_url, 'CloneFax', self.api_credentials, uuid=uuid, user_ip=user_ip,
                       user_agent=user_agent)
        return _get(self.http, url)

    def create(self, user_ip=IP_ADDR, user_agent=USER_AGENT, origin=ORIGIN):
        """Creates a new fax in the API backend and returns it.

        If a fax job is currently in edit mode in this session, this fax job is returned instead.
        Note: This is not an error. It provides you with the possibility to continue with the fax.

        Keyword arguments:
        user_ip -- The IP address of the client (if available). Put your own IP address otherwise.
        user_agent -- User agent string of the client device. If not available, put something descriptive (like "iPhone OS 2.2")
        origin -- From where was this fax started? i.e. "printer", "desktop", "home", ... For reporting and analysis purposes.

        """
        url = _get_url(self.base_url, 'Create', self.api_credentials, user_ip=user_ip, user_agent=user_agent,
                       origin=origin)
        return _get(self.http, url)

    def get_fax_state(self):
        """Returns the state of the current fax."""
        url = _get_url(self.base_url, 'GetFaxState', self.api_credentials)
        return _get(self.http, url)

    def get_preview(self):
        """Returns the states of all preview pages."""
        url = _get_url(self.base_url, 'GetPreview', self.api_credentials)
        return _get(self.http, url)

    def list_available_covers(self):
        """Returns a list of all coverpages the user may use.
        Result includes the "no cover" if the fax job already contains a file as in that case
        there's no need to add a cover.
        """
        url = _get_url(self.base_url, 'ListAvailableCovers', self.api_credentials)
        return _get(self.http, url)

    def list_fax_files(self):
        """Get all uploaded files for the current fax"""
        url = _get_url(self.base_url, 'ListFaxFiles', self.api_credentials)
        return _get(self.http, url)

    def list_recipients(self, current_page=None, items_per_page=None):
        """Returns the recipients for the current fax.

        Keyword arguments:
        current_page -- The page which should be shown
        items_per_page -- How many items are shown per page

        """
        url = _get_url(self.base_url, 'ListRecipients', self.api_credentials, current_page=current_page,
                       items_per_page=items_per_page)
        return _get(self.http, url)

    def remove_all_files(self):
        """Remove all uploaded files from the current fax"""
        url = _get_url(self.base_url, 'RemoveAllFiles', self.api_credentials)
        return _get(self.http, url)

    def remove_all_recipients(self):
        """Removes all recipients for the current fax."""
        url = _get_url(self.base_url, 'RemoveAllRecipients', self.api_credentials)
        return _get(self.http, url)

    def remove_cover(self):
        """Removes the cover from fax"""
        url = _get_url(self.base_url, 'RemoveCover', self.api_credentials)
        return _get(self.http, url)

    def remove_file(self, file_uuid):
        """Remove a file from the current fax."""
        url = _get_url(self.base_url, 'RemoveFile', self.api_credentials, file_uuid=file_uuid)
        return _get(self.http, url)

    def remove_recipient(self, number):
        """Removes a recipient from the current fax"""
        url = _get_url(self.base_url, 'RemoveRecipient', self.api_credentials, number=number)
        return _get(self.http, url)

    def send(self, send_at=None):
        """Start the fax sending.

        Only successful if all necessary data is set to the fax: at least 1 recipient and a cover page or a file uploaded.
        Will only work if user has enough credit to pay for the fax.
        You may pass in a datetime when the fax shall be sent. This must be a string formatted in the users chosen culture
        (so exactly as you would show it to him) and may not be in the past nor be greater than 'now + 14days'.

        """
        url = _get_url(self.base_url, 'Send', self.api_credentials, send_at=send_at)
        return _get(self.http, url)

    def send_delayed_fax_now(self, uuid):
        """Send a previously delayed fax now.

        Use this method if you want to send a fax right now that was initially delayed (by giving a send_at value into Send).

        """
        url = _get_url(self.base_url, 'SendDelayedFaxNow', self.api_credentials, uuid=uuid)
        return _get(self.http, url)

    def send_later(self, send_at=None):
        """Put the fax in the unpaid faxes queue.

        Only possible if user has NOT enough credit to send this fax directly.
        You may pass in a datetime when the fax shall be sent. This must be a string formatted in the users chosen culture
        (so exactly as you would show it to him) and may not be in the past nor be greater than 'now + 14days'.

        """
        url = _get_url(self.base_url, 'SendLater', self.api_credentials, send_at=send_at)
        return _get(self.http, url)

    def send_unpaid(self):
        """DEPRECATED! Use SendUnpaidFaxes instead"""
        return None

    def send_unpaid_faxes(self, uuids):
        """Send unpaid faxes

        Will work until credit reaches zero.
        Will return two lists: SentFaxes and UnpaidFaxes that contain the
        faxes that could or not be sent.

        """
        url = _get_url(self.base_url, 'SendUnpaidFaxes', self.api_credentials, uuids=uuids)
        return _get(self.http, url)

    def set_cover(self, template_id, text=None):
        """Sets the cover template for the current fax."""
        url = _get_url(self.base_url, 'SetCover', self.api_credentials, template_id=template_id, text=text)
        return _get(self.http, url)

    def set_notifications(self, notifications, group_notification=None, error_notification=None, save_defaults=None):
        """Sets the notification options for the current fax.

        Notification options that are not in the array will not be changed/resetted.
        Note: defaults for notification settings will be taken from users account, so potentially not need
        to call this on every fax.

        Keyword arguments:
        save_defaults -- Save Notification-Settings to user's Profile

        """
        url = _get_url(self.base_url, 'SetNotifications', self.api_credentials, notifications=notifications,
                       group_notification=group_notification, error_notification=error_notification,
                       save_defaults=save_defaults)
        return _get(self.http, url)

    def set_recipients(self, numbers, names=None):
        """Creates recipients for the current fax.

        All recipients are replaced with the given ones!

        """
        url = _get_url(self.base_url, 'SetRecipients', self.api_credentials, numbers=numbers, names=names)
        return _get(self.http, url)

    def start_preview_creation(self):
        """Starts creating the preview for this fax.

        Call after fax is ready (GetFaxState returns FAX_READY_TO_SEND)

        """
        url = _get_url(self.base_url, 'StartPreviewCreation', self.api_credentials)
        return _get(self.http, url)


# ----------------------------------------------------------------------------
# NumberInfo
# ----------------------------------------------------------------------------

class NumberInfo:
    """Class encapsulating information for a given fax number"""

    def __init__(self, api_credentials, http):
        """Instantiates the NumberInfo class"""
        self.base_url = '/NumberInfo'
        self.api_credentials = api_credentials
        self.http = http

    def get_number_info(self, faxnumber):
        """Get some information about a fax number.

        Result contains zone, type, city, ...
        Validates and corrects the number too.

        Arguments:
        faxnumber -- The faxnumber to query (incl countrycode: +12139851886, min length: 8)

        """
        url = _get_url(self.base_url, 'GetNumberInfo', self.api_credentials, faxnumber=faxnumber)
        return _get(self.http, url)

    def get_page_price(self, faxnumber):
        """Calculate the expected price per page to a given fax number.

        Use GetNumberInfo when you do not need pricing information, as calculating expected price takes longer then just looking up the info for a number.

        Arguments:
        faxnumber -- The faxnumber to query (incl countrycode: +12139851886, min length: 8). Login user first to get personalized prices.

        """
        url = _get_url(self.base_url, 'GetPagePrice', self.api_credentials, faxnumber=faxnumber)
        return _get(self.http, url)


# ----------------------------------------------------------------------------
# OnlineStorage
# ----------------------------------------------------------------------------

class OnlineStorage:
    """Class encapsulating actions related to online storage"""

    def __init__(self, api_credentials, http):
        """Instantiates the OnlineStorage class"""
        self.base_url = '/OnlineStorage'
        self.api_credentials = api_credentials
        self.http = http

    # DEPRECATED, instead do as written here: https://sandbox-apifrontend.pamfax.biz/processors/onlinestorage/
    # def authenticate(self, provider, username, password):

    def check_provider_state(self, provider):
        """NEW Return info about current provider state.

        Arguments:
        provider -- Provider name, see list_providers, e.g. DropBoxStorage
        """
        url = _get_url(self.base_url, 'CheckProviderState', self.api_credentials, provider=provider)
        return _get(self.http, url)

    def drop_authentication(self, provider):
        """Will drop the users authentication for the given provider.

        This will permanently erase all data related to the account!

        Arguments:
        provider -- Provider name, see list_providers, e.g. DropBoxStorage
        """
        url = _get_url(self.base_url, 'DropAuthentication', self.api_credentials, provider=provider)
        return _get(self.http, url)

    def get_provider_logo(self, provider, size):
        """Outputs a providers logo in a given size.

        Call ListProviders for valid sizes per Provider.

        Arguments:
        provider -- Provider name, see list_providers, e.g. DropBoxStorage
        size -- Icon size, see list_providers result, e.g. 32
        """
        url = _get_url(self.base_url, 'GetProviderLogo', self.api_credentials, provider=provider, size=size)
        return _get(self.http, url)

    def list_folder_contents(self, provider, folder=None, clear_cache=None):
        """Lists all files and folders inside a given folder.

        Leave folder empty to get the contents of the root folder.
        User must be autheticated for the provider given here to be able to recieve listings (see Authenticate method).
        If clear_cache is set to true all subitems will be deleted and must be refetched (previously cached UUIDs are invalid)

        Arguments:
        provider -- Provider name, see list_providers, e.g. DropBoxStorage
        folder -- Leave empty to get the contents of the root folder
        clear_cache -- If clear_cache is set to true all subitems will be deleted and must be refetched (previously cached UUIDs are invalid)
        """
        url = _get_url(self.base_url, 'ListFolderContents', self.api_credentials, provider=provider, folder=folder,
                       clear_cache=clear_cache)
        return _get(self.http, url)

    def list_providers(self, attach_settings=None):
        """Returns a list of supported providers."""
        url = _get_url(self.base_url, 'ListProviders', self.api_credentials, attach_settings=attach_settings)
        return _get(self.http, url)

    def set_auth_token(self, provider, token, username=None):
        """Manually sets auth token for the current user.

        token must contain an associative array including the tokens.
        sample (google):
        'auth_token_cp'=>DQAAAHoAAADCrkpB7Ip_vuelbla2UKE9s_ObVKTNA_kT6Ej26SwddJvMUmEz_9qbLlZJnsAdm583Sddp_0FYS9QmmwoUpf51RHxkgPUL20OqsdAP5OnCgY_TdVbvXX8tMQBBX30V4_NhTcE_0sI6zhba5Y3yZWV5nljliG98eA36ybekKucuhQ
        'auth_token_writely'=>DQAAAHoAAADCrkpB7Ip_vuelbla2UKE9s_ObVKTNA_kT6Ej62SDwdJvMUmEz_9qbLlZJnsAdm583Sddp_0FYS9QmmwoUpf51RHxkgPUL20OqsdAP5OnCgY_TdVbvXX8tMQBBX30V4_NhTcE_0sI6zhba5Y3yZWV5nljliG98eA36ybekKucuhQ
        'auth_token_wise'=>DQAAAHoAAADCrkpB7Ip_vuelbla2UKE9s_ObVKTNA_kT6Ej26SDdwJvMUmEz_9qbLlZJnsAdm583Sddp_0FYS9QmmwoUpf51RHxkgPUL20OqsdAP5OnCgY_TdVbvXX8tMQBBX30V4_NhTcE_0sI6zhba5Y3yZWV5nljliG98eA36ybekKucuhQ
        'auth_token_lh2'=>DQAAAHoAAADCrkpB7Ip_vuelbla2UKE9s_ObkvTNA_kT6Ej26SDwdJvMUmEz_9qbLlZJnsAdm583Sddp_0FYS9QmmwoUpf51RHxkgPUL20OqsdAP5OnCgY_TdVbvXX8tMQBBX30V4_NhTcE_0sI6zhba5Y3yZWV5nljliG98eA36ybekKucuhQ

        Arguments:
        provider -- Provider name
        token -- Associative array with token information
        username -- Optional username (for displaying purposes)

        """
        url = _get_url(self.base_url, 'SetAuthToken', self.api_credentials, provider=provider, token=token,
                       username=username)
        return _get(self.http, url)


# ----------------------------------------------------------------------------
# Session
# ----------------------------------------------------------------------------

class Session:
    """Class encapsulating a PamFax session"""

    def __init__(self, api_credentials, http):
        """Instantiates the Session class."""
        self.base_url = '/Session'
        self.api_credentials = api_credentials
        self.http = http

    def create_login_identifier(self, user_ip=None, timetolifeminutes=None):
        """Creates an identifier for the current user, which then can be passed to the portal to directly log in the user: https://portal.pamfax.biz/?_id=

        Be aware that these identifiers are case sensitive. Identifiers with ttl > 0 can only be used once.

        Keyword arguments:
        user_ip -- The IP address of the client on which this identifier will be bound to. Using the identfier from a different ip address will fail
        timetolifeminutes -- Optional a lifetime of this identifier. Defaults to 60 seconds. If <= 0 is given, the identifier does not expire and can be used more then once, but are tied to your current API key and can not be passed to online shop, portal, ... in the url

        """
        url = _get_url(self.base_url, 'CreateLoginIdentifier', self.api_credentials, user_ip=user_ip,
                       timetolifeminutes=timetolifeminutes)
        return _get(self.http, url)

    def list_changes(self):
        """Returns all changes in the system that affect the currently logged in user. This could be changes to the user's profile, credit, settings, ...

        Changes will be deleted after you received them once via this call, so use it wisely ;)

        """
        url = _get_url(self.base_url, 'ListChanges', self.api_credentials)
        return _get(self.http, url)

    def logout(self):
        """Terminate the current session. Log out."""
        url = _get_url(self.base_url, 'Logout', self.api_credentials)
        return _get(self.http, url)

    def ping(self):
        """Just keeps a session alive. If there is no activity in a Session for 5 minutes, it will be terminated.

        You then would need to call Session::VerifyUser again and start a new FaxJob

        """
        url = _get_url(self.base_url, 'Ping', self.api_credentials)
        return _get(self.http, url)

    def register_listener(self, listener_types, append=None):
        """Registers listeners for the current session. Any change of the listened types will then be available via Session::ListChanges function

        Arguments:
        listener_types -- Array of types to be registered ('faxall','faxsending','faxsucceeded','faxfailed','faxretrying')

        """
        url = _get_url(self.base_url, 'RegisterListener', self.api_credentials, listener_types=listener_types,
                       append=append)
        return _get(self.http, url)

    def reload_user(self):
        """Returns the current user object.

        Use this if you need to ensure that your locally stored user
        object is up to date.

        """
        url = _get_url(self.base_url, 'ReloadUser', self.api_credentials)
        return _get(self.http, url)

    def verify_user(self, username, password):
        """Verifies a user via username/password

        Arguments:
        username -- Username of the user or the md5 of user's username. That's what he has entered when he registered
        password -- The password (or the md5 of the password) that the user entered in the registration process for the given username (case sensitive)

        """
        url = _get_url(self.base_url, 'VerifyUser', self.api_credentials, username=username, password=password)
        return _get(self.http, url)


# ----------------------------------------------------------------------------
# Shopping
# ----------------------------------------------------------------------------

class Shopping:
    """Class encapsulating shopping options available through PamFax"""

    def __init__(self, api_credentials, http):
        """Instantiates the Shopping class"""
        self.base_url = '/Shopping'
        self.api_credentials = api_credentials
        self.http = http

    def add_credit_to_sandbox_user(self, amount, reason=None):
        """Adds some credit in user's currency to the currently logged in user.

        NOTES:
        This method is for testing purposes only, so it is not available in the LIVE system. Use it to create
        some 'virtual' credits for your users.

        Arguments:
        amount -- The amount of credit you want to add to the current user's pamfax credit. Currency is the user's currency. You can also pass a negative value to remove credit from user for testing.

        Keyword arguments:
        reason -- Optionally add some reason why you added this credit

        """
        url = _get_url(self.base_url, 'AddCreditToSandboxUser', self.api_credentials, amount=amount, reason=reason)
        return _get(self.http, url)

    def get_invoice(self, payment_uuid, hidecopy=None):
        """Returns an invoice pdf file for a payment (see UserInfo/ListOrders).

        Returns binary data so call with API_FORMAT_PASSTHRU.

        """
        url = _get_url(self.base_url, 'GetInvoice', self.api_credentials, payment_uuid=payment_uuid, hidecopy=hidecopy)
        return _get(self.http, url)

    def get_nearest_fax_in_number(self, ip_address):
        """Get the nearest available fax-in area code for the given IP-Address.

        Used to show "You can get a fax-in number in ..." to the user to offer PamFax plans to him

        Arguments:
        ip_address -- IP-Address to find nearest number for

        """
        url = _get_url(self.base_url, 'GetNearestFaxInNumber', self.api_credentials, ip_address=ip_address)
        return _get(self.http, url)

    def get_shop_link(self, type=None, product=None, pay=None):
        """Returns different shop links.

        Use these links to open a browser window with the shop in a specific state.

        Keyword arguments:
        type -- Type of link to return. Available: '', credit_packs, basic_plan, pro_plan, checkout
        product -- Product to add. Available: '',BasicPlan12,ProPlan12,OnDemand,Pack10,Pack30,Pack50,Pack100,Pack250,Pack500,Pack1000
        pay -- (DEPRECATED) direct leads to checkout page

        """
        url = _get_url(self.base_url, 'GetShopLink', self.api_credentials, type=type, product=product, pay=pay)
        return _get(self.http, url)

    def list_available_items(self):
        """Returns a list of available items from the shop.

        When a user is logged in, it will also contain the items only available to validated customers.

        """
        url = _get_url(self.base_url, 'ListAvailableItems', self.api_credentials)
        return _get(self.http, url)

    def list_fax_in_areacodes(self, country_code, state=None):
        """Returns available fax-in area codes in a given country+state"""
        url = _get_url(self.base_url, 'ListFaxInAreacodes', self.api_credentials, country_code=country_code,
                       state=state)
        return _get(self.http, url)

    def list_fax_in_countries(self):
        """Retruns a list of countries where new fax-in numbers are currently available"""
        url = _get_url(self.base_url, 'ListFaxInCountries', self.api_credentials)
        return _get(self.http, url)

    def redeem_credit_voucher(self, vouchercode):
        """Redeem a credit voucher.

        These are different then the shop vouchers to be used in the online shop!
        You can use "PCPC0815" to test this function in the Sandbox API

        Arguments:
        vouchercode -- The voucher code. Format is ignored.

        """
        url = _get_url(self.base_url, 'RedeemCreditVoucher', self.api_credentials, vouchercode=vouchercode)
        return _get(self.http, url)


# ----------------------------------------------------------------------------
# UserInfo
# ----------------------------------------------------------------------------

class UserInfo:
    """Class encapsulating user info"""

    def __init__(self, api_credentials, http):
        """Instantiates the UserInfo class"""
        self.base_url = '/UserInfo'
        self.api_credentials = api_credentials
        self.http = http

    def create_user(self, name, username, password, email, culture, externalprofile=None, campaign_id=None):
        """Create a new PamFax user and logs him in

        Arguments:
        name -- First and last name of the user
        username -- Unique username. Call will fail with bad_username error if the username already exists. Min 6 chars, max 60 chars. Can be left empty if externalprofile is given. Then the username will be generated from externalprofile. You could use UserInfo::ValidateNewUsername to create a unique username first and then pass it to this call
        password -- a password for the user. If left empty, a new password will be generated. Min 8 chars, max 20 chars. If length is 32, it's assumed as md5 of original password. Please then check min and max length by yourself!
        email -- A valid email address for this user.
        culture -- The culture of the user (en-US, de-DE, ...)

        Keyword arguments:
        externalprofile -- External profile data. externalprofile["type"] is the type of data: skype. externalprofile["client_ip"] should be set to the user's ip address
        campaign_id -- is used to payout recommendation bonus to given user uuid. i.e if user clicked on links like http://www.pamfax.biz/?ref=b36d019d53ba, the b36d019d53ba should be passed as campaign_id

        """
        url = _get_url(self.base_url, 'CreateUser', self.api_credentials, name=name, username=username,
                       password=password, email=email, culture=culture, externalprofile=externalprofile,
                       campaign_id=campaign_id)
        return _get(self.http, url)

    def delete_user(self):
        """Deletes the currently logged in users account.

        All assigned numbers and data will be deleted too!
        Warning: User will be deleted permanently without any chance to recover his data!

        """
        url = _get_url(self.base_url, 'DeleteUser', self.api_credentials)
        return _get(self.http, url)

    def get_culture_info(self):
        """Returns the users culture information"""
        url = _get_url(self.base_url, 'GetCultureInfo', self.api_credentials)
        return _get(self.http, url)

    def has_plan(self):
        """Check if the user has a Plan.

        This would NOT include other fax numbers user has access to. Will return NONE if no plan, PRO or BASIC otherwise

        """
        url = _get_url(self.base_url, 'HasPlan', self.api_credentials)
        return _get(self.http, url)

    def list_expirations(self, type=None):
        """Returns expirations from current user

        if type==false all Expirations are Returned
        else CREDIT, PROPLAN and

        """
        url = _get_url(self.base_url, 'ListExpirations', self.api_credentials, type=type)
        return _get(self.http, url)

    def list_inboxes(self, expired_too=None, shared_too=None):
        """Return the inboxes of the user with some additional data (like expiration).

        Keyword arguments:
        expired_too -- If true, lists all expired numbers too.

        """
        url = _get_url(self.base_url, 'ListInboxes', self.api_credentials, expired_too=expired_too,
                       shared_too=shared_too)
        return _get(self.http, url)

    def list_orders(self, current_page=None, items_per_page=None):
        """Returns a list of orders for this user"""
        url = _get_url(self.base_url, 'ListOrders', self.api_credentials, current_page=current_page,
                       items_per_page=items_per_page)
        return _get(self.http, url)

    def list_profiles(self):
        """Read the full profile of the user."""
        url = _get_url(self.base_url, 'ListProfiles', self.api_credentials)
        return _get(self.http, url)

    def list_user_agents(self, max=None):
        """Returns a list of user-agents the user has used to sent faxes.

        List will be sorted by amount of faxes sent, so it is a top max list.

        Keyword arguments:
        max -- How many results (5-20, default 5)

        """
        url = _get_url(self.base_url, 'ListUserAgents', self.api_credentials, max=max)
        return _get(self.http, url)

    def list_wall_messages(self, count=None, data_to_list=None):
        """Returns a list of activities for the user

        This list contains report about what has been happened lately in users account
        (like messages sent to the user, faxes sent, faxes received, orders placed, ...)

        Keyword arguments:
        count -- The count of items to return. Valid values are between 1 and 100
        data_to_list -- Any message types you want this function to return. Allowed models are 'faxsent', 'faxin', 'faxout', 'payment', 'message' and 'news'. Leave empty to get messages of any type.

        """
        url = _get_url(self.base_url, 'ListWallMessages', self.api_credentials, count=count, data_to_list=data_to_list)
        return _get(self.http, url)

    def send_message(self, body, type=None, recipient=None, subject=None):
        """Send a message to the user

        Arguments:
        body -- The message body

        Keyword arguments:
        type -- Type of message to send. Currently implemented: email, skypechat or sms
        recipient -- Recipient of the message. Might be an email address, IM username or phone number depending on the message type
        subject -- Optionally a subject. Not used in all message types (likely not used in SMS and chat)

        """
        url = _get_url(self.base_url, 'SendMessage', self.api_credentials, body=body, type=type, recipient=recipient,
                       subject=subject)
        return _get(self.http, url)

    def send_password_reset_message(self, username, user_ip=None):
        """Send a password reset message to a user

        Arguments:
        username -- PamFax username to send the message to

        """
        url = _get_url(self.base_url, 'SendPasswordResetMessage', self.api_credentials, username=username,
                       user_ip=user_ip)
        return _get(self.http, url)

    def set_online_storage_settings(self, provider, settings):
        """Sets users OnlineStorage settings.

        Expects the settings to be given as key-value pairs.
        Currently supported settings are:
            - inbox_enabled: Store incoming faxes (0|1, required)
            - inbox_path: Path to store incoming faxes to (path as string folders separated by '/', leading and trailing '/' optional, required)

        Arguments:
        provider -- Provider store settings for (see OnlineStorageApi::ListProviders)
        settings -- Key-value pairs of settings.

        """
        url = _get_url(self.base_url, 'SetOnlineStorageSettings', self.api_credentials, provider=provider,
                       settings=settings)
        return _get(self.http, url)

    def set_password(self, password, hashFunction=None, old_password=None):
        """Set a new login password for the currently logged in user.

        You may use md5 encrypted passwords by setting the value of hashFunction to 'md5'.
        Note: password values must be lower case when using a hashFunction other than 'plain'!

        Arguments:
        password -- The new password. Needs to be between 6 and 60 chars long

        Keyword arguments:
        hashFunction -- The function used to enrycpt the password. Allowed values: plain or md5.
        old_password -- The current password. This is optional for the moment but will be required in future versions. Note that the $hashFunction value applies to this argument too.

        """
        url = _get_url(self.base_url, 'SetPassword', self.api_credentials, password=password, hashFunction=hashFunction,
                       old_password=old_password)
        return _get(self.http, url)

    def set_profile_properties(self, profile, properties, ignoreerrors=None):
        """Saves values to an extended user profile

        Users may have different profiles. Use this method to store values in them.
        Profiles will be created if not present yet.

        Arguments:
        properties -- key=>value pairs of all profiles properties to save

        Keyword arguments:
        ignoreerrors -- If a field can not be found in the profile object, just ignore it. Otherwise returns an error

        """
        url = _get_url(self.base_url, 'SetProfileProperties', self.api_credentials, profile=profile,
                       properties=properties, ignoreerrors=ignoreerrors)
        return _get(self.http, url)

    def validate_new_username(self, username, dictionary=None):
        """Validate a username for a new user.

        Returns a list of suggestions for the username if given username is already in use. Call this prior to UserInfo/CreateUser to show alternative usernames to the user if the entered username is already occupied or invalid.

        Arguments:
        username -- Unique username to validate. Call will fail with bad_username error if the username already exists. Min 6 chars, max 60 chars.

        """
        url = _get_url(self.base_url, 'ValidateNewUsername', self.api_credentials, username=username,
                       dictionary=dictionary)
        return _get(self.http, url)



#!/usr/bin/python3

import os
import random
import time

from pamfax import PamFax

# Either configure it manually or by using an external config.py file
if os.path.isfile('./config.py'):
    from config import HOST, USERNAME, PASSWORD, APIKEY, APISECRET
else:
    HOST = 'api.pamfax.biz'  # Either (testing) sandbox-api.pamfax.biz OR (production) api.pamfax.biz
    USERNAME = 'your PamFax username here'
    PASSWORD = 'your PamFax password here'
    APIKEY = 'your PamFax API key here'
    APISECRET = 'your PamFax API secret word here'


def _assert_json(response):
    assert response['result']['code'] == 'success'


if __name__ == "__main__":

    print('Welcome to the PamFax Python 3 Sample by using the a1pamfax package\n')
    print('Host: ' + HOST)
    print('Username: ' + USERNAME)
    print('API Key: ' + APIKEY)

    print("\nLogin with PamFax credentials..")
    pamfax = PamFax(USERNAME, PASSWORD, host=HOST, apikey=APIKEY, apisecret=APISECRET)

    print("Creating a fax with cover..")
    response = pamfax.create()
    _assert_json(response)
    response = pamfax.list_available_covers()
    _assert_json(response)
    response = pamfax.set_cover(response['Covers']['content'][1]['id'], 'My test fax with PamFax using Python3')
    _assert_json(response)

    print("Adding a random recipient..")
    rnd_phone = str(random.randint(10000000, 99999999))  # Berlin has 8 digits after 030..
    response = pamfax.add_recipient('+49030' + rnd_phone)
    _assert_json(response)

    print("Waiting until fax is ready to send..")
    while True:
        response = fax_state = pamfax.get_state()
        _assert_json(response)
        # print(fax_state['FaxContainer']['state'])
        if fax_state['FaxContainer']['state'] == 'ready_to_send':
            break
        time.sleep(2)

    print("Fax setup done, sending..")
    response = pamfax.send()
    _assert_json(response)

    input("\nPress ENTER twice to exit")



#!/usr/bin/python3

"""
This module implements the PamFax API*. It's a rewrite of the
dynaptico's pamfax api in Python 2 - to work now for Python 3.

See the following link for more details on the PamFax API:

https://sandbox-apifrontend.pamfax.biz/processors/

NOTE: This module is for and has only been tested under Python 3.6+.

Following signatures have changed to the older (Python 2) implementation:

_get_and_check_response
_get
_post

* CAUTION: Dropbox methods are not implemented (yet).
"""

import logging
import time
import types
from urllib.parse import urlencode

from .processors import Common, FaxHistory, FaxJob, NumberInfo, OnlineStorage, Session, Shopping, UserInfo, \
    _get, _get_url

logger = logging.getLogger('pamfax')
try:
    logger.addHandler(logging.NullHandler())
except:
    pass
logger.setLevel(logging.DEBUG)


class PamFax:
    """Class encapsulating the PamFax API. Actions related to the sending of faxes are called on objects of this class.
    For example, the 'create' action resides in the FaxJob class, but you can just use the following 'shortcut' logic:

    from pamfax import PamFax
    p = PamFax(<args>)
    p.create()

    NOTE: To keep signatures backward compatible like the older implementation in Python 2, the http parameter
    is used now to transport the host.
    """

    def __init__(self, username, password, host='api.pamfax.biz', apikey='', apisecret=''):
        """Creates an instance of the PamFax class and initiates an HTTPS session."""
        logger.info("Connecting to %s", host)
        http = host  # Previously HTTPSConnection(host=host, port=443, timeout=142)
        api_credentials = '?%s' % urlencode(
            {'apikey': apikey, 'apisecret': apisecret, 'apioutputformat': 'API_FORMAT_JSON'})
        usertoken = self._get_user_token(host, api_credentials, username, password)
        api_credentials = '%s&%s' % (api_credentials, urlencode({'usertoken': usertoken}))

        common = Common(api_credentials, http)
        fax_history = FaxHistory(api_credentials, http)
        fax_job = FaxJob(api_credentials, http)
        number_info = NumberInfo(api_credentials, http)
        online_storage = OnlineStorage(api_credentials, http)
        session = Session(api_credentials, http)
        shopping = Shopping(api_credentials, http)
        user_info = UserInfo(api_credentials, http)

        attrs = dir(self)
        for processor in (session, common, fax_history, fax_job, number_info, online_storage, shopping, user_info):
            for attr_key in dir(processor):
                if attr_key not in attrs:
                    attr_value = getattr(processor, attr_key)
                    if isinstance(attr_value, types.MethodType):
                        setattr(self, attr_key, attr_value)

    def _verify_user(self, http, api_credentials, username, password):
        """Verifies a user via username/password"""
        url = _get_url('/Session', 'VerifyUser', api_credentials, username=username, password=password)
        return _get(http, url)

    def _get_user_token(self, http, api_credentials, username, password):
        """Gets the user token to use with subsequent requests."""
        result = self._verify_user(http, api_credentials, username, password)
        if result['result']['code'] == 'success':
            return result['UserToken']['token']
        else:
            raise Exception(result['result']['message'])

    # ------------------------------------------------------------------------
    # Convenient helper methods
    # ------------------------------------------------------------------------

    def get_state(self, blocking=False, interval=1):
        """Obtains the state of the FaxJob build, may block until a state is received, or just return immediately"""
        if blocking:
            state = None
            result = None
            while state is None:
                result = self.get_fax_state()
                time.sleep(interval)
            return result
        else:
            return self.get_fax_state()

    def is_converting(self, fax_state):
        """Returns whether or not a file in the fax job is still in a converting state."""
        converting = False
        files = fax_state['Files']
        if 'content' in files:
            for f in files['content']:
                state = f['state']
                if state == '' or state == 'converting':
                    converting = True
        return converting


if __name__ == '__main__':
    print("This is the Python 3 implementation of the PamFax API.\n\n"
          "To run the test suite, please run:\n\n"
          "  cd test\n"
          "  python test.py")



#!/usr/bin/python3

# Save this file as config.py and adapt (the copy!) to your credentials!
HOST = 'sandbox-api.pamfax.biz'
USERNAME = '<email-or-username>'
PASSWORD = '<password>'
APIKEY = '<api-username>'
APISECRET = '<api-secret>'

# ToDo: DEPRECATED, to be removed (also import statements from source)
DROPBOX_USERNAME = '<dropbox-username>'
DROPBOX_PASSWORD = '<dropbox-password>'


#!/usr/bin/python3

import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="a1pamfax",
    version="0.0.7",
    author="Marc Bufe (Airport1)",
    author_email="bufemc@gmail.com",
    description="Python 3 implementation for the PamFax API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license='LICENSE.txt',
    url="http://github.com/bufemc/a1pamfax/",
    packages=['pamfax', 'pamfax.processors'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
)
